;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                                         ;;;
;;;  Model of PWI-PRP task as described in                                  ;;;
;;;  Van Maanen, Van Rijn, & Taatgen, Experiment 1                          ;;;
;;;                                                                         ;;;
;;;  Leendert van Maanen                                                    ;;;
;;;  University of Groningen                                                ;;;
;;;  leendert@ai.rug.nl                                                     ;;;
;;;                                                                         ;;;
;;;  090711                                                                 ;;;
;;;                                                                         ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(clear-all)

(setf *act-r-enabled-p* t)


(defvar *perceive* t)

(defparameter *RT1Con* nil)
(defparameter *RT1Rel* nil)
(defparameter *RT1Unrel* nil)
(defparameter *RT2Con* nil)
(defparameter *RT2Rel* nil)
(defparameter *RT2Unrel* nil)

(defparameter *RT1ConTotal* nil)
(defparameter *RT1RelTotal* nil)
(defparameter *RT1UnrelTotal* nil)
(defparameter *RT2ConTotal* nil)
(defparameter *RT2RelTotal* nil)
(defparameter *RT2UnrelTotal* nil)

(defparameter *activations* nil)

(defparameter *am-start* nil)
(defparameter *condition* nil)
(defparameter *done-am* nil)
(defparameter *done-stroop* nil)
(defparameter *done* nil)

(defparameter *response-time* nil)
(defparameter *response* nil)
(defparameter *responses* nil)

(defparameter *exp-stim* '((("groen" red "rood" "R") 100) (("groen" red "rood" "R") 350) (("groen" red "rood" "R") 800)
                           (("XXXXX" red "rood" "U") 100) (("XXXXX" red "rood" "U") 350) (("XXXXX" red "rood" "U") 800)))

(defun do-trial (soa trial window &optional (trialnr nil))
  (setf *perceive* t)
  (setf *done* nil)
  (setf *done-am* nil)
  (setf *done-stroop* nil)
  (goal-focus am)
  (setf *condition* (fourth trial))
  
  (let ((tone (act-r-random 2))
        (start-time nil))
    
    (cond
     ((> soa 0)
      (new-tone-sound (case tone (0 300) (1 600) (2 1200)) .3 (mp-time))
      (setf *am-start* (get-time))

      (schedule-event-relative (/ soa 1000) 'add-exp-txt :params trial)
      (setf start-time (+ (get-time) soa)))
   
     ((< soa 0)      
      (new-tone-sound (case tone (0 300) (1 600) (2 1200)) .3 (/ (abs soa) 1000))
      (setf *am-start* (+ (get-time) (abs soa)))
    
      (setf start-time (get-time))
      (add-text-to-exp-window :text (first trial) :x 130 :y 110 :width 60 :color (second trial))))
        
    (when *actr-enabled-p*
      (proc-display)
      
      (run 15))
    (while (null *done*) 
      (allow-event-manager window))
    (clear-exp-window)
    (proc-display)
    (if *response-time*
      (cond ((equalp (fourth trial) "C")
          (push (list trialnr soa (- *response-time* start-time) *response*) *RT2Con*))
            ((equalp (fourth trial) "U")
             (push (list trialnr soa (- *response-time* start-time) *response*) *RT2Unrel*))
            (t
             (push (list trialnr soa (- *response-time* start-time) *response*) *RT2Rel*))))
    (push *response* *responses*)))

(defun add-exp-txt (text color na na2)
  (declare (ignore na))
  (declare (ignore na2))
  (add-text-to-exp-window :text text :x 130 :y 110 :width 60 :color color)
  (proc-display)  
)

(defun do-1-trial (soa cond)
  "runs one trial. soa is an integer, cond is one of \"R\", \"C\", or \"U\"" 
  (reset)
  (setf *responses* nil)
  (setf *activations* nil)
  (let ((window (open-exp-window "PRP Stroop"                                      
                                 :visible nil
                                 :width 300
                                 :height 300))) 
    (if *act-r-enabled-p* (install-device window))
    
    (setf *RT1Con* nil)
    (setf *RT1Rel* nil)
    (setf *RT1Unrel* nil)
    (setf *RT2Con* nil)
    (setf *RT2Rel* nil)
    (setf *RT2Unrel* nil)
    (do-trial 
     soa 
     (cond ((equalp cond "C") '("rood" red "rood" "C"))
           ((equalp cond "R") '("groen" red "rood" "R"))
           (t '("XXXXX" red "rood" "U")))
     window)
    (cond ((equalp cond "C") (format t "RT1: ~a, RT2: ~a~%" *RT1Con* *RT2Con*))
           ((equalp cond "R") (format t "RT1: ~a, RT2: ~a~%" *RT1Rel* *RT2Rel*))
           (t (format t "RT1: ~a, RT2: ~a~%" *RT1Unrel* *RT2Unrel*)))
  ))

(defun do-exp (&optional (pps 10) (filename "test") (fixtime 50))
  "Runs a sequence of trials. each condition soa combination occurs pps times. fixtime 
is the time between subsequent trials in seconds."
  (setf *responses* nil)
  (setf *RT1ConTotal* nil)
  (setf *RT1RelTotal* nil)  
  (setf *RT1UnrelTotal* nil)
  (setf *RT2ConTotal* nil)
  (setf *RT2RelTotal* nil)
  (setf *RT2UnrelTotal* nil)
  (dotimes (pp pps t)
    (write-data filename)
    (setf *responses* nil)
    (setf *RT1ConTotal* nil)
    (setf *RT1RelTotal* nil)  
    (setf *RT1UnrelTotal* nil)
    (setf *RT2ConTotal* nil)
    (setf *RT2RelTotal* nil)
    (setf *RT2UnrelTotal* nil)
    (setf *RT1Con* nil)
    (setf *RT1Rel* nil)
    (setf *RT1Unrel* nil)
    (setf *RT2Con* nil)
    (setf *RT2Rel* nil)
    (setf *RT2Unrel* nil)
    (reset)
    (let ((window (open-exp-window "PRP PWI"                                      
                                   :visible nil
                                   :width 300
                                   :height 300))
          (i 0))
      (if *act-r-enabled-p* (install-device window))
      (dolist (stim (permute-list *exp-stim*))
        (incf i 1)
        (format t "pp ~a, trial ~a, cond ~a~%" pp i stim)
        (schedule-break-relative fixtime :details "wait for new trial")
        (setf (race-last-rt (get-module race)) nil)
        (do-trial (cadr stim) (car stim) window i)
        (reset-declarative-finsts)
        )
      (push (list pp *RT1Con*) *RT1ConTotal*)
      (push (list pp *RT1Rel*) *RT1RelTotal*)
      (push (list pp *RT1Unrel*) *RT1UnrelTotal*)
      (push (list pp *RT2Con*) *RT2ConTotal*)
      (push (list pp *RT2Rel*) *RT2RelTotal*)
      (push (list pp *RT2Unrel*) *RT2UnrelTotal*)))
  (if filename (write-data filename)))
      
(defun set-vision-state-to-free ()
  (let ((vm (get-module :vision)))
    (reset-pm-module vm)
    (change-state vm :prep 'free :exec 'free :proc 'free)
    ))

(defun set-retrieval-state-to-free ()
  (let ((dm (get-module declarative)))
    (setf (dm-failed dm) nil)
    ))

;;devices

(defmethod device-speak-string ((win rpm-window) text)
  (setf *response-time* (get-time))
  (setf *response* text)
  (setf *done-stroop* t)
  (if (and *done-am* *done-stroop*) (setf *done* t)))

(defmethod rpm-window-key-event-handler ((win rpm-window) key)
 ; (format t "~%Key press: gettime: ~a, *start: ~a~%key: ~a~%" (get-time) *am-start* key)
  (cond ((equalp *condition* "C")
      (push (list -1 -1 (- (get-time) *am-start*) key) *RT1Con*))
        ((equalp *condition* "R")
         (push (list -1 -1 (- (get-time) *am-start*) key) *RT1Rel*))
        (t
         (push (list -1 -1 (- (get-time) *am-start*) key) *RT1Unrel*)))
  (setf *done-am* t)
  (if (and *done-am* *done-stroop*) (setf *done* t)))

(defmethod queue-output-events ((spch-mod speech-module) (mvmt speak))
;  (new-word-sound (text mvmt) (+ (mp-time) (exec-time mvmt)))
  (queue-command
   :time (exec-time mvmt) :where :DEVICE :command 'output-speech
   :params (text mvmt)
   ;;; Dan
   :from :speech))

;; write to file

(defparameter *output-pathname* (current-pathname))
(defun write-data (filename)
  (with-open-file (stream (ensure-directories-exist 
                        (merge-pathnames 
                         (make-pathname :name filename :type "txt")  
                         *output-pathname*)) :direction :output :if-exists :append :if-does-not-exist :create)
    (dolist (trials *RT1ConTotal*)
      (mapcar #'(lambda (x) (format stream "RT1 Con ~a ~a ~a ~a ~a~%" (car trials) (second x) (third x) (first x) (fourth x))) (cadr trials)))
    (dolist (trials *RT1RelTotal*)
      (mapcar #'(lambda (x) (format stream "RT1 Rel ~a ~a ~a ~a ~a~%" (car trials) (second x) (third x) (first x) (fourth x))) (cadr trials)))
    (dolist (trials *RT1UnrelTotal*)
      (mapcar #'(lambda (x) (format stream "RT1 Unrel ~a ~a ~a ~a ~a~%" (car trials) (second x) (third x) (first x) (fourth x))) (cadr trials)))
    (dolist (trials *RT2ConTotal*)
      (mapcar #'(lambda (x) (format stream "RT2 Con ~a ~a ~a ~a ~a~%" (car trials) (second x) (third x) (first x) (fourth x))) (cadr trials)))
    (dolist (trials *RT2RelTotal*)
      (mapcar #'(lambda (x) (format stream "RT2 Rel ~a ~a ~a ~a ~a~%" (car trials) (second x) (third x) (first x) (fourth x))) (cadr trials)))
    (dolist (trials *RT2UnrelTotal*)
      (mapcar #'(lambda (x) (format stream "RT2 Unrel ~a ~a ~a ~a ~a~%" (car trials) (second x) (third x) (first x) (fourth x))) (cadr trials)))
    ))

(defparameter *wordpopout* 2.5)
(defparameter *rand-tone-time* 4.0) ; higher: more variance
(defparameter *lf-m* 0.2770079) ; mean of the deadline distribution. i.e. a deadline of 2500 ms given a rt of -2.2.
(defparameter *lf-s* 0.04) ; scale of the deadline dist. a scale of 440ms given a rt of -2.2.

(define-model PRP-PWI

(sgp
 :v nil
 :trace-detail low
 :act nil
 :tone-detect-delay 0.049
 :tone-recode-delay 0.385 :alpha 0.05 :egs 0.04 ; 
 :VISUAL-ATTENTION-LATENCY 0.00 ; because we use RACE to control this
 :MOTOR-FEATURE-PREP-TIME .04
 :MOTOR-INITIATION-TIME .04
 :randomize-time 1 
 :vpft t ; production time noise

 :bll 0.5
 :esc t
 :ol t ; to use the hybrid learning equation (petrov 2006) set to 1
 :rt -2.2

 :ans .43
 :pas nil
 :race-enabled t
 :race-ratio .975
 :race-freq 200
 :race-a .65
 :race-b 0.7
 :race-saliency .55
 :race-default-acc 0
 :race-trace nil

 :save-buffer-trace t
 :traced-buffers (visual aural retrieval manual vocal production)
)

;;aural manual declarations
(chunk-type stimresp tone key)
(chunk-type aural-vocal expect state)

(add-dm
 (s-r1 isa stimresp tone 300 key "j")
 (s-r2 isa stimresp tone 600 key "k")
 (s-r3 isa stimresp tone 1200 key "l")

 (am isa aural-vocal expect concept state "start")
 (concept isa chunk)
)

(sdp s-r1 :creation-time -100000 :reference-count 1000)
(sdp s-r2 :creation-time -100000 :reference-count 1000)
(sdp s-r3 :creation-time -100000 :reference-count 1000)

(add-sji
 (s-r1 s-r2 0)
 (s-r1 s-r3 0)
 (s-r3 s-r1 0)
 (s-r2 s-r1 0)
 (s-r3 s-r2 0)
 (s-r2 s-r3 0)
)

;;PWI declarations
(chunk-type lemma value concept)
(chunk-type concept value)
(chunk-type wordform value)
(chunk-type naming expect state)
(chunk-type reading expect state)

(add-dm
 (red-lemma isa lemma value "rood" concept red)
 (green-lemma isa lemma value "groen" concept green)
 (xxxxx-lemma isa lemma value "xxxxx" concept "nil")
 (red-concept isa concept value red)
 (green-concept isa concept value green)
 (black-concept isa concept value black)
 (red-wordform isa wordform value "rood")
 (green-wordform isa wordform value "groen")
 (xxxxx-wordform isa wordform value "xxxxx")

 (naming isa naming expect concept state "start")
 (reading isa reading expect lemma state "start")
 (lemma isa chunk)
)

(sdp red-lemma :creation-time -47000 :reference-count 100)
(sdp green-lemma :creation-time -47000 :reference-count 100)
(sdp xxxxx-lemma :creation-time -47000 :reference-count 100)
(sdp red-concept :creation-time -47000 :reference-count 100)
(sdp green-concept :creation-time -47000 :reference-count 100)
(sdp black-concept :creation-time -47000 :reference-count 100)

(sdp red-wordform :creation-time -47000 :reference-count 100)
(sdp green-wordform :creation-time -47000 :reference-count 100)
(sdp xxxxx-wordform :creation-time -47000 :reference-count 100)

(add-sji
 (red-wordform red-lemma .02)
 (green-wordform green-lemma .02)
 (xxxxx-wordform xxxxx-lemma 0.02)

 (red-lemma red-concept .01)
 (red-concept red-lemma .06)

 (green-lemma green-concept .01)
 (green-concept green-lemma .06)

 (red-concept green-concept .5)
 (green-concept red-concept .5)
)

;;; Aural Manual task

(p detect-tone
  =goal>
   isa aural-vocal
  =aural-location>
    isa audio-event
  ?aural>
    state free
==>
  +aural>
    isa sound
    event =aural-location
    !eval! (setf (dm-lf (get-module declarative)) (max 0 (+ (act-r-noise *lf-s*) *lf-m*)))
)

(p attend-tone-and-respond-300
  =goal>
     isa aural-vocal
  =aural>
    isa sound
    content 300
  ?manual>
    state free
==>
  +manual>
   isa press-key
   key "j"
  +goal>
    isa naming
    expect concept
    state "busy"
  !eval! (set-retrieval-state-to-free)
)

(p attend-tone-and-respond-600
  =goal>
     isa aural-vocal
  =aural>
    isa sound
    content 600
  ?manual>
    state free
==>
  +manual>
   isa press-key
   key "k"
  +goal>
    isa naming
    expect concept
    state "busy"
  !eval! (set-retrieval-state-to-free)
)

(p attend-tone-and-respond-1200
  =goal>
     isa aural-vocal
  =aural>
    isa sound
    content 1200
  ?manual>
    state free
==>
  +manual>
   isa press-key
   key "l"
  +goal>
    isa naming
    expect concept
    state "busy"
  !eval! (set-retrieval-state-to-free)
)

;;; naming task

(p perceive-stimulus-naming
   =visual-location>
      isa visual-location
   ?visual>
      state free
      buffer empty
   !eval! *perceive*
   ==>
   +visual>               
      isa      move-attention
      screen-pos =visual-location
   =visual-location>
   !eval! (setf *perceive* nil)
)

(p retrieve-lemma
   =visual>
      isa concept
   =goal>
      isa naming
      - state "stop"
    ?retrieval>
      state free
      buffer empty
   ==>
   +retrieval>
      isa lemma
      :recently-retrieved nil
   =visual>
)
 

(p reject-lemma-naming
   =retrieval>
      isa lemma
      concept =color
   =visual-location>
      isa visual-location
      - color =color
      color =viscolor
   =goal>
      isa naming
      - state "stop"
    ?retrieval>
      state free
   ==>
   +retrieval>
      isa lemma
      concept =viscolor
      :recently-retrieved nil
   =visual>
   =visual-location>
)
(spp reject-lemma-naming :u 1)
; the utility values are needed to make sure that usually reject-lemma-naming fires 
; before encode-response-naming. This acts as a form of cognitive control (checking 
; the correctness of the color lemma). Noise on the utilities ensures that sometimes
; encode-response-naming will be selected, which is a source of error.

(p encode-response-naming
   =retrieval>
      isa lemma
      value =val
   =visual-location>
      isa visual-location
   =goal>
      isa naming
      - state "stop"
    ?retrieval>
      state free
   ==>
   +retrieval>
      isa wordform
      value =val
)
(spp encode-response-naming :u .83)

(p pwi-error
   =goal>
      isa naming
      - state "stop"   
   ?retrieval>
      state error
==>
   !eval! (set-vision-state-to-free)
   !eval! (set-retrieval-state-to-free)
   +vocal>
      isa speak
      string "error"
   =goal>
      state "stop"
   -visual>
   !output! "error"
   !eval! (setf *done* t)
)

(p respond-pwi
   =retrieval>
      isa wordform
      value =val
   ?vocal>
      state free
   =goal>
      isa naming
      - state "stop"
   ==>
   !eval! (set-vision-state-to-free)
   !eval! (set-retrieval-state-to-free)
   +vocal>
      isa speak
      string =val
   =goal>
      state "stop"
   -visual>
   -retrieval>
)

)